@@ -1,9 +1,10 @@
 import uuid
+from collections.abc import Callable, Sequence
 from concurrent.futures import Future, ThreadPoolExecutor, as_completed
 from functools import partial
 from os import PathLike
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Optional, Sequence, Union
+from typing import Any
 
 import httpx
 
@@ -17,35 +18,26 @@ MiB = 1024 * 1024
 class Resumable:
     """A resumable-httpx upload client.
 
-    Parameters
-    ----------
-    target : str
-        The URL of the resumable upload target
-    chunk_size : int, optional
-        The size, in bytes, of file chunks to be uploaded
-    simultaneous_uploads : int, optional
-        The number of file chunk uploads to attempt at once
-    headers : dict, optional
-        A dictionary of additional HTTP headers to include in requests
-    test_chunks : bool
-        Flag indicating if the client should check with the server if a chunk
-        already exists with a GET request prior to attempting to upload the
-        chunk with a POST
-    max_chunk_retries : int, optional
-        The number of times to retry uploading a chunk
-    permanent_errors : collection of int, optional
-        HTTP status codes that indicate the upload of a chunk has failed and
-        should not be retried
+    Args:
+        target: The URL of the resumable upload target
+        chunk_size: The size, in bytes, of file chunks to be uploaded. Defaults to 1MB.
+        simultaneous_uploads: The number of file chunk uploads to attempt at once.
+            Defaults to 3.
+        headers: A dictionary of additional HTTP headers to include in requests.
+            Defaults to None.
+        test_chunks: Flag indicating if the client should check with the server if a chunk
+            already exists with a GET request prior to attempting to upload the chunk with
+            a POST. Defaults to True.
+        max_chunk_retries: The number of times to retry uploading a chunk.
+            Defaults to 100.
+        permanent_errors: HTTP status codes that indicate the upload of a chunk has failed
+            and should not be retried. Defaults to (400, 404, 415, 500, 501).
 
-    Attributes
-    ----------
-    file_added : resumable.util.CallbackDispatcher
-        Triggered when a file has been added, passing the file object
-    file_completed : resumable.util.CallbackDispatcher
-        Triggered when a file upload has completed, passing the file object
-    chunk_completed : resumable.util.CallbackDispatcher
-        Triggered when a chunk upload has completed, passing the file and chunk
-        objects
+    Attributes:
+        file_added: Triggered when a file has been added, passing the file object.
+        file_completed: Triggered when a file upload has completed, passing the file object.
+        chunk_completed: Triggered when a chunk upload has completed, passing the file and
+            chunk objects.
     """
 
     def __init__(
@@ -53,14 +45,13 @@ class Resumable:
         target: str,
         chunk_size: int = MiB,
         simultaneous_uploads: int = 3,
-        headers: Optional[Dict[str, Any]] = None,
+        headers: dict[str, Any] | None = None,
         test_chunks: bool = True,
         max_chunk_retries: int = 100,
         permanent_errors: Sequence[int] = (400, 404, 415, 500, 501),
-        query: Optional[Dict[str, Any]] = None,
-        generate_unique_identifier: Callable[
-            [Path, Path], str
-        ] = lambda _path, _relative_path: str(uuid.uuid4()),
+        query: dict[str, Any] | None = None,
+        generate_unique_identifier: Callable[[Path, Path], str] = lambda _path,
+        _relative_path: str(uuid.uuid4()),
         client: httpx.Client = httpx.Client(),
     ) -> None:
         if headers is None:
@@ -83,19 +74,18 @@ class Resumable:
         self.client = client
         self.client.headers.update(headers)
 
-        self.files: List[ResumableFile] = []
+        self.files: list[ResumableFile] = []
 
         self.executor = ThreadPoolExecutor(simultaneous_uploads)
-        self.futures: List[Future] = []
+        self.futures: list[Future] = []
 
         self.file_added = CallbackDispatcher()
         self.file_completed = CallbackDispatcher()
         self.chunk_completed = CallbackDispatcher()
 
     def add_file(
-        self, path: Union[PathLike, str], relative_path: Optional[Path]
+        self, path: PathLike | str, relative_path: Path | None
     ) -> ResumableFile:
-
         file = ResumableFile(
             Path(path),
             relative_path,
