@@ -45,30 +45,29 @@ There are the following four options to specify which server context to use:
    subsequent interactions in the same python run as well.
 """
 
-
 import os
 from contextlib import ContextDecorator
 from contextvars import ContextVar, Token
-from functools import lru_cache
-from typing import Any, List, Optional
+from functools import cache
+from typing import Any
 
 import attr
 from dotenv import load_dotenv
 from rich.prompt import Prompt
 
-from webknossos.client._defaults import DEFAULT_HTTP_TIMEOUT, DEFAULT_WEBKNOSSOS_URL
-from webknossos.client._generated import Client as GeneratedClient
+from ._defaults import DEFAULT_HTTP_TIMEOUT, DEFAULT_WEBKNOSSOS_URL
+from .api_client import DatastoreApiClient, WkApiClient
 
 load_dotenv()
 
 
-@lru_cache(maxsize=None)
+@cache
 def _cached_ask_for_token(webknossos_url: str) -> str:
-    # TODO  pylint: disable=fixme
+    # TODO # noqa: FIX002 Line contains TODO
     # -validate token and ask again if necessary
     # -ask if the token should be saved in some .env file
     # -reset invalid tokens
-    #  (e.g. use cachetools for explicit cache managment:
+    #  (e.g. use cachetools for explicit cache management:
     #  https://cachetools.readthedocs.io/en/stable/#memoizing-decorators)
     return Prompt.ask(
         f"\nPlease enter your webknossos token as shown on {webknossos_url}/auth/token ",
@@ -76,55 +75,46 @@ def _cached_ask_for_token(webknossos_url: str) -> str:
     )
 
 
-@lru_cache(maxsize=None)
+@cache
 def _cached_get_org(context: "_WebknossosContext") -> str:
-    from webknossos.client._generated.api.default import current_user_info
-
-    current_user_info_response = current_user_info.sync(
-        client=context.generated_auth_client
-    )
-    assert current_user_info_response is not None
-    return current_user_info_response.organization
+    current_api_user = context.api_client_with_auth.user_current()
+    return current_api_user.organization
 
 
-# TODO reset invalid tokens e.g. using cachetools  pylint: disable=fixme
-@lru_cache(maxsize=None)
+# TODO reset invalid tokens e.g. using cachetools # noqa: FIX002 Line contains TODO
+@cache
 def _cached_get_datastore_token(context: "_WebknossosContext") -> str:
-    from webknossos.client._generated.api.default import generate_token_for_data_store
-
-    generate_token_for_data_store_response = generate_token_for_data_store.sync(
-        client=context.generated_auth_client
-    )
-    assert generate_token_for_data_store_response is not None
-    return generate_token_for_data_store_response.token
+    api_datastore_token = context.api_client_with_auth.token_generate_for_data_store()
+    return api_datastore_token.token
 
 
-@lru_cache(maxsize=None)
-def _cached__get_generated_client(
+@cache
+def _cached__get_api_client(
     webknossos_url: str,
-    token: Optional[str],
+    token: str | None,
     timeout: int,
-) -> GeneratedClient:
+) -> WkApiClient:
     """Generates a client which might contain an x-auth-token header."""
     if token is None:
-        return GeneratedClient(base_url=webknossos_url, timeout=timeout)
-    else:
-        return GeneratedClient(
-            base_url=webknossos_url, headers={"X-Auth-Token": token}, timeout=timeout
-        )
+        return WkApiClient(base_wk_url=webknossos_url, timeout_seconds=timeout)
+    return WkApiClient(
+        base_wk_url=webknossos_url,
+        headers={"X-Auth-Token": token},
+        timeout_seconds=timeout,
+    )
 
 
 def _clear_all_context_caches() -> None:
     _cached_ask_for_token.cache_clear()
     _cached_get_org.cache_clear()
     _cached_get_datastore_token.cache_clear()
-    _cached__get_generated_client.cache_clear()
+    _cached__get_api_client.cache_clear()
 
 
 @attr.frozen
 class _WebknossosContext:
     url: str = os.environ.get("WK_URL", default=DEFAULT_WEBKNOSSOS_URL).rstrip("/")
-    token: Optional[str] = os.environ.get("WK_TOKEN", default=None)
+    token: str | None = os.environ.get("WK_TOKEN", default=None)
     timeout: int = int(os.environ.get("WK_TIMEOUT", default=DEFAULT_HTTP_TIMEOUT))
 
     # all properties are cached outside to allow re-usability
@@ -148,7 +138,7 @@ class _WebknossosContext:
         return _cached_get_org(self)
 
     @property
-    def datastore_token(self) -> Optional[str]:
+    def datastore_token(self) -> str | None:
         if self.token is None:
             return None
         else:
@@ -159,17 +149,22 @@ class _WebknossosContext:
         return _cached_get_datastore_token(self)
 
     @property
-    def generated_client(self) -> GeneratedClient:
-        return _cached__get_generated_client(self.url, self.token, self.timeout)
+    def api_client(self) -> WkApiClient:
+        return _cached__get_api_client(self.url, self.token, self.timeout)
 
     @property
-    def generated_auth_client(self) -> GeneratedClient:
-        return _cached__get_generated_client(
-            self.url, self.required_token, self.timeout
-        )
+    def api_client_with_auth(self) -> WkApiClient:
+        return _cached__get_api_client(self.url, self.required_token, self.timeout)
 
-    def get_generated_datastore_client(self, datastore_url: str) -> GeneratedClient:
-        return GeneratedClient(base_url=datastore_url, timeout=self.timeout)
+    def get_datastore_api_client(self, datastore_url: str) -> DatastoreApiClient:
+        headers = None
+        if self.datastore_token is not None:
+            headers = {"X-Auth-Token": self.datastore_token}
+        return DatastoreApiClient(
+            datastore_base_url=datastore_url,
+            timeout_seconds=self.timeout,
+            headers=headers,
+        )
 
 
 _webknossos_context_var: ContextVar[_WebknossosContext] = ContextVar(
@@ -186,35 +181,47 @@ class webknossos_context(ContextDecorator):
 
     def __init__(
         self,
-        url: Optional[str] = None,
-        token: Optional[str] = None,
-        timeout: Optional[int] = None,
+        url: str | None = None,
+        token: str | None = None,
+        timeout: int | None = None,
     ) -> None:
-        """Returns a new WEBKNOSSOS server contextmanager. Use with the `with` statement:
-        ```python
-        with webknossos_context(token="my_webknossos_token"):
-            # code that interacts with webknossos
-        ```
+        """Creates a new WEBKNOSSOS server context manager.
 
-        Or as a decorator:
-        ```python
-        @webknossos_context(token="my_webknossos_token")
-        def my_func():
-            # code that interacts with webknossos
-        ```
+        Can be used as a context manager with 'with' or as a decorator.
 
-        You can specify the following arguments:
-        * `url`, by default [https://webknossos.org](https://www.webknossos.org),
-        * `token`, as displayed on [https://webknossos.org/auth/token](https://webknossos.org/auth/token),
-        * `timeout` to specify a custom network request timeout in seconds, `1800` (30min) by default.
+        Args:
+            url: Base URL for WEBKNOSSOS server, defaults to https://webknossos.org.
+                Taken from previous context if not specified.
+            token: Authentication token from https://webknossos.org/auth/token.
+                Must be specified explicitly.
+            timeout: Network request timeout in seconds, defaults to 1800 (30 min).
+                Taken from previous context if not specified.
 
-        `url` and `timeout` are taken from the previous context (e.g. environment variables) if not specified.
-        `token` must be set explicitly, it is not available when not specified.
+        Examples:
+            Using as context manager:
+                ```
+                with webknossos_context(token="my_webknossos_token"):
+                    # code that interacts with webknossos
+                    ds.download(...)
+                ```
+
+            Using as decorator:
+                ```
+                @webknossos_context(token="my_webknossos_token")
+                def my_func():
+                    # code that interacts with webknossos
+                    ...
+                ```
+
+        Note:
+            The url and timeout parameters will use values from the previous context
+            (e.g. environment variables) if not specified explicitly. The token
+            parameter must always be set explicitly.
         """
         self._url = _get_context().url if url is None else url.rstrip("/")
         self._token = token
         self._timeout = _get_context().timeout if timeout is None else timeout
-        self._context_var_token_stack: List[Token[_WebknossosContext]] = []
+        self._context_var_token_stack: list[Token[_WebknossosContext]] = []
 
     def __enter__(self) -> None:
         context_var_token = _webknossos_context_var.set(
@@ -226,8 +233,8 @@ class webknossos_context(ContextDecorator):
         _webknossos_context_var.reset(self._context_var_token_stack.pop())
 
 
-def _get_generated_client(enforce_auth: bool = False) -> GeneratedClient:
+def _get_api_client(enforce_auth: bool = False) -> WkApiClient:
     if enforce_auth:
-        return _get_context().generated_auth_client
+        return _get_context().api_client_with_auth
     else:
-        return _get_context().generated_client
+        return _get_context().api_client
