@@ -1,23 +1,24 @@
 import logging
 import math
 import warnings
+from collections.abc import Callable
 from enum import Enum
 from itertools import product
-from typing import TYPE_CHECKING, Callable, List, Optional, Tuple
+from typing import TYPE_CHECKING, Optional
 
 import numpy as np
 from scipy.ndimage import zoom
 
 if TYPE_CHECKING:
-    from .dataset import Dataset, Layer
-
-from webknossos.geometry import Mag, Vec3Int, Vec3IntLike
-from webknossos.geometry.bounding_box import BoundingBox
+    from .dataset import Dataset
 
+from ..geometry import Mag, Vec3Int, Vec3IntLike
 from ._array import ArrayInfo
 from .layer_categories import LayerCategoryType
 from .view import View
 
+logger = logging.getLogger(__name__)
+
 
 class InterpolationModes(Enum):
     MEDIAN = 0
@@ -29,19 +30,25 @@ class InterpolationModes(Enum):
     MIN = 6
 
 
-DEFAULT_BUFFER_SHAPE = Vec3Int.full(256)
+def determine_downsample_buffer_shape(array_info: ArrayInfo) -> Vec3Int:
+    # This is the shape of the data in the downsampling target magnification, so the
+    # data that is read is up to 512³ vx in the source magnification. Using larger
+    # shapes uses a lot of RAM, especially for segmentation layers which use the mode filter.
+    # See https://scm.slack.com/archives/CMBMU5684/p1749771929954699 for more context.
+    return Vec3Int.full(256).pairmin(array_info.shard_shape)
 
 
-def determine_buffer_shape(array_info: ArrayInfo) -> Vec3Int:
-    return min(DEFAULT_BUFFER_SHAPE, array_info.shard_shape)
+def determine_upsample_buffer_shape(array_info: ArrayInfo) -> Vec3Int:
+    # This is the shape of the data in the upsampling target magnification.
+    return array_info.shard_shape
 
 
 def calculate_mags_to_downsample(
     from_mag: Mag,
     coarsest_mag: Mag,
     dataset_to_align_with: Optional["Dataset"],
-    voxel_size: Optional[Tuple[float, float, float]],
-) -> List[Mag]:
+    voxel_size: tuple[float, float, float] | None,
+) -> list[Mag]:
     assert np.all(from_mag.to_np() <= coarsest_mag.to_np())
     mags = []
     current_mag = from_mag
@@ -54,15 +61,15 @@ def calculate_mags_to_downsample(
             for mag in layer.mags.keys()
         )
     mags_to_align_with_by_max_dim = {mag.max_dim: mag for mag in mags_to_align_with}
-    assert len(mags_to_align_with) == len(
-        mags_to_align_with_by_max_dim
-    ), "Some layers contain different values for the same mag, this is not allowed."
+    assert len(mags_to_align_with) == len(mags_to_align_with_by_max_dim), (
+        "Some layers contain different values for the same mag, this is not allowed."
+    )
     while current_mag < coarsest_mag:
         if current_mag.max_dim * 2 in mags_to_align_with_by_max_dim:
             current_mag = mags_to_align_with_by_max_dim[current_mag.max_dim * 2]
             if current_mag > coarsest_mag:
                 warnings.warn(
-                    "The mag taken from another layer is larger in some dimensions than coarsest_mag."
+                    "[INFO] The mag taken from another layer is larger in some dimensions than `coarsest_mag`."
                 )
         elif voxel_size is None:
             # In case the sampling mode is CONSTANT_Z or ISOTROPIC:
@@ -95,7 +102,7 @@ def calculate_mags_to_downsample(
                 )
                 # In case of isotropic resolution but anisotropic mag we need to ensure unique max dims.
                 # current mag: 4-4-2, voxel_size: 1,1,1 -> new_mag: 4-4-4, therefore we skip this entry.
-                if new_mag.max_dim == current_mag.max_dim:
+                if new_mag.max_dim == current_mag.max_dim and new_mag != current_mag:
                     current_mag = new_mag
                     continue
             if new_mag == current_mag:
@@ -113,8 +120,8 @@ def calculate_mags_to_upsample(
     from_mag: Mag,
     finest_mag: Mag,
     dataset_to_align_with: Optional["Dataset"],
-    voxel_size: Optional[Tuple[float, float, float]],
-) -> List[Mag]:
+    voxel_size: tuple[float, float, float] | None,
+) -> list[Mag]:
     return list(
         reversed(
             calculate_mags_to_downsample(
@@ -138,7 +145,6 @@ def parse_interpolation_mode(
     interpolation_mode: str, layer_category: LayerCategoryType
 ) -> InterpolationModes:
     if interpolation_mode.upper() == "DEFAULT":
-
         return (
             InterpolationModes.MODE
             if layer_category == "segmentation"
@@ -148,7 +154,7 @@ def parse_interpolation_mode(
         return InterpolationModes[interpolation_mode.upper()]
 
 
-def linear_filter_3d(data: np.ndarray, factors: List[int], order: int) -> np.ndarray:
+def linear_filter_3d(data: np.ndarray, factors: list[int], order: int) -> np.ndarray:
     factors_np = np.array(factors)
 
     return zoom(
@@ -167,10 +173,10 @@ def linear_filter_3d(data: np.ndarray, factors: List[int], order: int) -> np.nda
 
 
 def non_linear_filter_3d(
-    data: np.ndarray, factors: List[int], func: Callable[[np.ndarray], np.ndarray]
+    data: np.ndarray, factors: list[int], func: Callable[[np.ndarray], np.ndarray]
 ) -> np.ndarray:
     ds = data.shape
-    assert not any((d % factor > 0 for (d, factor) in zip(ds, factors)))
+    assert not any(d % factor > 0 for (d, factor) in zip(ds, factors))
     data = data.reshape((ds[0], factors[1], ds[1] // factors[1], ds[2]), order="F")
     data = data.swapaxes(0, 1)
     data = data.reshape(
@@ -251,7 +257,7 @@ def _mode(x: np.ndarray) -> np.ndarray:
     # Find maximum counts and return modals/counts
     slices = [slice(None, i) for i in sort.shape]
     del slices[axis]
-    index = np.ogrid[slices]
+    index = list(np.ogrid[slices])
     index.insert(axis, np.argmax(counts, axis=axis))
     return sort[tuple(index)]
 
@@ -279,7 +285,7 @@ def downsample_unpadded_data(
 
 
 def downsample_cube(
-    cube_buffer: np.ndarray, factors: List[int], interpolation_mode: InterpolationModes
+    cube_buffer: np.ndarray, factors: list[int], interpolation_mode: InterpolationModes
 ) -> np.ndarray:
     if interpolation_mode == InterpolationModes.MODE:
         return non_linear_filter_3d(cube_buffer, factors, _mode)
@@ -296,11 +302,11 @@ def downsample_cube(
     elif interpolation_mode == InterpolationModes.MIN:
         return non_linear_filter_3d(cube_buffer, factors, _min)
     else:
-        raise Exception("Invalid interpolation mode: {}".format(interpolation_mode))
+        raise Exception(f"Invalid interpolation mode: {interpolation_mode}")
 
 
 def downsample_cube_job(
-    args: Tuple[View, View, int],
+    args: tuple[View, View, int],
     mag_factors: Vec3Int,
     interpolation_mode: InterpolationModes,
     buffer_shape: Vec3Int,
@@ -309,30 +315,32 @@ def downsample_cube_job(
 
     try:
         num_channels = target_view.info.num_channels
-        shape = (num_channels,) + target_view.bounding_box.in_mag(
-            target_view.mag
-        ).size.to_tuple()
+        target_bbox_in_mag = target_view.bounding_box.in_mag(target_view.mag)
+        shape = (num_channels,) + target_bbox_in_mag.size.to_tuple()
+        shape_xyz = target_bbox_in_mag.size_xyz
         file_buffer = np.zeros(shape, target_view.get_dtype())
 
         tiles = product(
             *(
-                list(range(0, math.ceil(len / buffer_edge_len)))
-                for len, buffer_edge_len in zip(shape[-3:], buffer_shape)
+                list(range(0, math.ceil(length / buffer_edge_len)))
+                for length, buffer_edge_len in zip(shape_xyz, buffer_shape)
             )
         )
 
         for tile in tiles:
             target_offset = Vec3Int(tile) * buffer_shape
-            source_offset = mag_factors * target_offset
-            source_size = source_view.bounding_box.in_mag(source_view.mag).size
-            source_size = (mag_factors * buffer_shape).pairmin(
+            source_offset = target_offset * target_view.mag
+            source_size = source_view.bounding_box.size_xyz
+            source_size = (buffer_shape * target_view.mag).pairmin(
                 source_size - source_offset
             )
 
-            bbox = BoundingBox(source_offset, source_size)
+            bbox = source_view.bounding_box.offset(source_offset).with_size_xyz(
+                source_size
+            )
 
-            cube_buffer_channels = source_view.read(
-                relative_bounding_box=bbox.from_mag_to_mag1(source_view.mag),
+            cube_buffer_channels = source_view.read_xyz(
+                absolute_bounding_box=bbox,
             )
 
             for channel_index in range(num_channels):
@@ -346,16 +354,22 @@ def downsample_cube_job(
                         interpolation_mode,
                     )
 
-                    buffer_bbox = BoundingBox(target_offset, data_cube.shape)
-                    file_buffer[(channel_index,) + buffer_bbox.to_slices()] = data_cube
+                    buffer_bbox = target_view.bounding_box.with_topleft_xyz(
+                        target_offset
+                    ).with_size_xyz(data_cube.shape)
+
+                    # Add missing axes to the data_cube if bbox is nd
+                    data_cube = buffer_bbox.xyz_array_to_bbox_shape(data_cube)
+
+                    file_buffer[(channel_index,) + buffer_bbox.to_slices_xyz()] = (
+                        data_cube
+                    )
 
         # Write the downsampled buffer to target
-        if source_view.info.num_channels == 1:
-            file_buffer = file_buffer[0]  # remove channel dimension
-        target_view.write(file_buffer)
+        target_view.write(file_buffer, absolute_bounding_box=target_view.bounding_box)
 
     except Exception as exc:
-        logging.error(
+        logger.error(
             f"Downsampling of target {target_view.bounding_box} failed with {exc}"
         )
         raise exc
