@@ -1,4 +1,4 @@
-from typing_extensions import Literal
+from typing import Literal
 
 COLOR_CATEGORY: Literal["color"] = "color"
 SEGMENTATION_CATEGORY: Literal["segmentation"] = "segmentation"
